# Load libraries
library(marked)
library(rgdal)
library(rgeos)

# Load data files
dataset <- read.csv("dataset.csv")
meadows <- readOGR("Meadows.gpkg", "Meadows")
proj4string(meadows) <- CRS("+proj=longlat +datum=WGS84 +no_defs")
meadows <- spTransform(meadows , CRS("+init=epsg:3035"))
meadows@data$Area <- round(gArea(meadows, byid = T))
meadows@data$MeadowID <- 1:nrow(meadows@data)

################################################
# Prepare MRR estiamtes using CJS model
################################################

formatMrr <- function(x, meadow_name) {
    dates <- unique(x$Date)
    individual.numbers <- unique(x$IndividualName)

    matrica <- matrix(nrow = length(individual.numbers), ncol = 3)
    matrica[, 2] <- individual.numbers
    for (j in 1:length(individual.numbers)) {

        individual.number <- individual.numbers[j]
        dataset.no <- dataset[dataset$IndividualName == individual.number, ]
        #print(paste0("Individual ", individual.number, "; sex: ", dataset.no$Sex[1]))
        matrica[j, 3] <- dataset.no$Sex[1]
        captures <- c()

        for (k in 1:length(dates)) {

            if (nrow(dataset.no[dataset.no$Date %in% dates[k], ]) == 0) {
                #print("Nema")
                captures <- paste0(captures, "0")
            } else {
                #print("Ima")
                captures <- paste0(captures, "1")
            }
            matrica[j, 1] <- captures
        }
    }

    time.interval <- c()
    for (i in 1:(length(dates)-1)) {
        interval <- dates[i+1] - dates[i]
        time.interval <- c(time.interval, interval)
        #print(interval)
    }

    colnames(matrica) <- c("ch", "no", "sex")
    matrica <- as.data.frame(matrica)
    return(list("matrix" = matrica, "time.interval" = time.interval, "dates" = dates, "meadow_name" = meadow_name))
}

# Function used to fit CJS model
fitMrr <- function(x) {
    m <- x$matrix
    t <- x$time.interval

    # Apparent survival - Phi
    #Phi.sex.time <- list(formula=~sex*time)
    #Phi.time <- list(formula=~time)
    Phi.sex <- list(formula=~sex)
    Phi.dot <- list(formula=~1)

    # Detection probability - p
    #p.sex.time <- list(formula=~sex*time)
    #p.time <- list(formula=~time)
    p.sex <- list(formula=~sex)
    p.dot <- list(formula=~1)

    # Process data (and set grouping variables)
    m$sex <- as.factor(m$sex)
    m.proc <- process.data(m, group = "sex", time.intervals = t)

    # Make design data (from processed data)
    m.ddl <- make.design.data(m.proc)

    # Construct all combinations and put into one model table
    cml <- create.model.list(c("Phi","p")) # makes all possibile combinations of those parameter formulas
    m.cjs.models <- crm.wrapper(cml, data = m.proc, ddl = m.ddl, external = FALSE,  accumulate = FALSE, hessian = TRUE)

    bestmodel <- as.numeric(rownames(m.cjs.models$model.table[1,]))

    m.cjs.best <- m.cjs.models[[bestmodel]]
    est.best <- predict(m.cjs.best, ddl = m.ddl, se = TRUE)
    est.simplest <- predict(m.cjs.models[[1]], ddl = m.ddl, se = TRUE)

    return(list("best_model" = est.best, "simplest.model" = est.simplest, "aic" = m.cjs.models, "MeadowName" = x$meadow_name))

}

# A function used to split matrix values into separate colums for each day
getNumberOfCaptures <- function(matrix, dates, sex) {
    if (is.na(sex)) {
        matrica.split <- strsplit(as.character(matrix$ch), "(?=.)", perl=TRUE)
    } else {
        matrica.split <- strsplit(as.character(matrix[matrix$sex == sex, ]$ch), "(?=.)", perl=TRUE)
    }
        matrica.split <- data.frame(do.call(rbind, matrica.split))
        colnames(matrica.split) <- dates
        number.of.captures <- apply(matrica.split,  2, function(y) {sum(as.numeric(y))})
        return(number.of.captures)
}

# A function used to calculate recruitment
getRecruitment <- function(estimates, data) {

    getRecruitmentEstimate <- function(dates, captures, p, Phi, time.intervals) {
        p <- as.numeric(p)
        Phi <- as.numeric(Phi)

        # Calculate recruitment
        r <- matrix(nrow = (length(dates) - 1), ncol = 2)
        daily.pop.size <- captures / p
        if (length(dates) >= 3) {
            for (j in 1:(length(dates) - 1)) {
                B <- daily.pop.size[(j+1)] - daily.pop.size[j] * Phi ^ time.intervals[j]
                r[j, 1] <- B
            }
            r[r[, 1] < 0, ] <- 0

            # Calculate adjusted recruitment by Nowicki et al. 2005
            for (j in 1:(length(dates) - 1)) {
                r[j, 2] <- time.intervals[j] * r[j, 1] * (Phi - 1) / ( Phi ^ time.intervals[j] - 1 )
            }
        } else {
            r <- rbind(c(NA, NA))
        }
            r <- as.data.frame(r)
            colnames(r) <- c("Recruitment", "AdjustedRecruitment")
            return(r)
    }

    recruitment <- list(length = length(estimates))
    for (i in 1:(length(estimates))) {
        data.t <- data[[i]]
        # Check if CJS model is fitted
        if (length(estimates[[i]]) == 0) {
            # Use average estimate for calculating recruitment if CJS not fitted
            p <- c()
            for (j in 1:length(estimates)) {
                p <- c(p, estimates[[j]]$best_model$p$estimate)
            }
            Phi <- c()
            for (j in 1:length(estimates)) {
                Phi <- c(Phi, estimates[[j]]$best_model$Phi$estimate)
            }

            no.captures <- getNumberOfCaptures(data.t$matrix, data.t$dates, NA)
            r <- getRecruitmentEstimate(data.t$dates, no.captures, mean(as.numeric(p)), mean(as.numeric(Phi)), data.t$time.interval)
            print(paste0(data.t$meadow_name, ": ", round(sum(r$Recruitment))))
            recruitment[[i]] <- list("MeadowName" = data.t$meadow_name, "Recruitment" = r$Recruitment, "RecruitmentMale" = NA, "RecruitmentFemale" = NA,
                "RecruitmentAdjusted" = r$AdjustedRecruitment, "RecruitmentAdjustedMale" = NA, "RecruitmentAdjustedFemale" = NA, "TotalPopulation" = round(sum(r$Recruitment)))
        } else {
            estimate <- estimates[[i]]$best_model
            if (nrow(estimate$p) == 1 & nrow(estimate$Phi) == 1) {
                no.captures <- getNumberOfCaptures(data.t$matrix, data.t$dates, NA)
                r <- getRecruitmentEstimate(data.t$dates, no.captures, estimate$p$estimate, estimate$Phi$estimate, data.t$time.interval)
                print(paste0(data.t$meadow_name, ": ", round(sum(r$Recruitment))))
                recruitment[[i]] <- list("MeadowName" = data.t$meadow_name, "Recruitment" = r$Recruitment, "RecruitmentMale" = NA, "RecruitmentFemale" = NA,
                    "RecruitmentAdjusted" = r$AdjustedRecruitment, "RecruitmentAdjustedMale" = NA, "RecruitmentAdjustedFemale" = NA, "TotalPopulation" = round(sum(r$Recruitment)))
            }

            if (nrow(estimate$p) == 2 & nrow(estimate$Phi) == 1) {
                no.captures.m <- getNumberOfCaptures(data.t$matrix, data.t$dates, "Male")
                no.captures.f <- getNumberOfCaptures(data.t$matrix, data.t$dates, "Female")
                r.m <- getRecruitmentEstimate(data.t$dates, no.captures.m, estimate$p[estimate$p$sex == "Male", ]$estimate, estimate$Phi$estimate, data.t$time.interval)
                r.f <- getRecruitmentEstimate(data.t$dates, no.captures.f, estimate$p[estimate$p$sex == "Female", ]$estimate, estimate$Phi$estimate, data.t$time.interval)
                print(paste0(data.t$meadow_name, ": ", round(sum(r.m$Recruitment)) + round(sum(r.f$Recruitment)), " (M = ", round(sum(r.m$Recruitment)), "; F = ", round(sum(r.f$Recruitment)), ")"))
                recruitment[[i]] <- list("MeadowName" = data.t$meadow_name, "Recruitment" = r.m$Recruitment + r.f$Recruitment, "RecruitmentMale" = r.m$Recruitment, "RecruitmentFemale" = r.f$Recruitment,
                    "RecruitmentAdjusted" = r.m$AdjustedRecruitment + r.f$AdjustedRecruitment, "RecruitmentAdjustedMale" = r.m$AdjustedRecruitment, "RecruitmentAdjustedFemale" = r.f$AdjustedRecruitment, "TotalPopulation" = round(sum(r.m$AdjustedRecruitment + r.f$AdjustedRecruitment)))
            }

            if (nrow(estimate$p) == 1 & nrow(estimate$Phi) == 2) {
                no.captures.m <- getNumberOfCaptures(data.t$matrix, data.t$dates, "Male")
                no.captures.f <- getNumberOfCaptures(data.t$matrix, data.t$dates, "Female")
                r.m <- getRecruitmentEstimate(data.t$dates, no.captures.m, estimate$p$estimate, estimate$Phi[estimate$Phi$sex == "Male", ]$estimate, data.t$time.interval)
                r.f <- getRecruitmentEstimate(data.t$dates, no.captures.f, estimate$p$estimate, estimate$Phi[estimate$Phi$sex == "Female", ]$estimate, data.t$time.interval)
                print(paste0(data.t$meadow_name, ": ", round(sum(r.m$Recruitment)) + round(sum(r.f$Recruitment)), " (M = ", round(sum(r.m$Recruitment)), "; F = ", round(sum(r.f$Recruitment)), ")"))
                recruitment[[i]] <- list("MeadowName" = data.t$meadow_name, "Recruitment" = r.m$Recruitment + r.f$Recruitment, "RecruitmentMale" = r.m$Recruitment, "RecruitmentFemale" = r.f$Recruitment,
                    "RecruitmentAdjusted" = r.m$AdjustedRecruitment + r.f$AdjustedRecruitment, "RecruitmentAdjustedMale" = r.m$AdjustedRecruitment, "RecruitmentAdjustedFemale" = r.f$AdjustedRecruitment, "TotalPopulation" = round(sum(r.m$AdjustedRecruitment + r.f$AdjustedRecruitment)))
            }

            if (nrow(estimate$p) == 2 & nrow(estimate$Phi) == 2) {
                no.captures.m <- getNumberOfCaptures(data.t$matrix, data.t$dates, "Male")
                no.captures.f <- getNumberOfCaptures(data.t$matrix, data.t$dates, "Female")
                r.m <- getRecruitmentEstimate(data.t$dates, no.captures.m, estimate$p[estimate$p$sex == "Male", ]$estimate, estimate$Phi[estimate$Phi$sex == "Male", ]$estimate, data.t$time.interval)
                r.f <- getRecruitmentEstimate(data.t$dates, no.captures.f, estimate$p[estimate$p$sex == "Female", ]$estimate, estimate$Phi[estimate$Phi$sex == "Female", ]$estimate, data.t$time.interval)
                print(paste0(data.t$meadow_name, ": ", round(sum(r.m$Recruitment)) + round(sum(r.f$Recruitment)), " (M = ", round(sum(r.m$Recruitment)), "; F = ", round(sum(r.f$Recruitment)), ")"))
                recruitment[[i]] <- list("MeadowName" = data.t$meadow_name, "Recruitment" = r.m$Recruitment + r.f$Recruitment, "RecruitmentMale" = r.m$Recruitment, "RecruitmentFemale" = r.f$Recruitment,
                    "RecruitmentAdjusted" = r.m$AdjustedRecruitment + r.f$AdjustedRecruitment, "RecruitmentAdjustedMale" = r.m$AdjustedRecruitment, "RecruitmentAdjustedFemale" = r.f$AdjustedRecruitment, "TotalPopulation" = round(sum(r.m$AdjustedRecruitment + r.f$AdjustedRecruitment)))
            }
        }
    }
    return(recruitment)
}

# Create separate datasets for each meadow to be used in CJS
dataset.mrr <- list(length = length(unique(dataset$MeadowName)))
for (i in 1:length(unique(dataset$MeadowName))) {
    meadow <- unique(dataset$MeadowName)[i]
    #print(paste0("Meadow ", meadow))
    meadow.data <- dataset[dataset$MeadowName == meadow, ]
    dataset.mrr[[i]] <- formatMrr(meadow.data, meadow)
}

# Remove meadows that could not be fitted due to small
# sample size and fit CJS for the rest of them.
estimates.mrr <- list(length = length(dataset.mrr))
for (i in c(1:length(dataset.mrr))) {
    print(paste0("Loop number ", i))

    try({ # Wrap in try to continue if error occures
        estimates.mrr[[i]] <- fitMrr(dataset.mrr[[i]])
    })
}

recruitment <- getRecruitment(estimates.mrr, dataset.mrr)

####################################################
# Prepare data for dispersal analysis
####################################################

getRecaptures <- function(x) {
    recaptures <- list()
    for (i in 1:length(unique(x$IndividualName))) {
        individual <- unique(x$IndividualName)[i]
        tttt <- x[x$IndividualName == individual, ]

        if (nrow(tttt) > 1) {
            dispersed <- c()
            dispersed.from <- c()
            distance <- c()
            for (j in 1:(nrow(tttt) - 1)) {

                # Get the distances betweens consecutive captures
                coordinate.before <- as.data.frame(matrix(nrow = 1, ncol = 2))
                coordinate.after <- as.data.frame(matrix(nrow = 1, ncol = 2))
                coordinates(coordinate.before) <- cbind(tttt$X[j], tttt$Y[j])
                coordinates(coordinate.after) <- cbind(tttt$X[j+1], tttt$Y[j+1])
                proj4string(coordinate.before) <- CRS("+proj=longlat +datum=WGS84 +no_defs")
                proj4string(coordinate.after) <- CRS("+proj=longlat +datum=WGS84 +no_defs")
                coordinate.before <- spTransform(coordinate.before, CRS("+init=epsg:3035"))
                coordinate.after <- spTransform(coordinate.after, CRS("+init=epsg:3035"))
                distance <- c(distance, gDistance(coordinate.before, coordinate.after))

                if (tttt$MeadowName[j] == tttt$MeadowName[j+1]) {
                    dispersed.from <- c(dispersed.from, tttt$MeadowName[j])
                    dispersed <- c(dispersed, "No")
                } else {
                    dispersed.from <- c(dispersed.from, tttt$MeadowName[j])
                    dispersed <- c(dispersed, "Yes")
                }
            }
            recaptures[[length(recaptures) + 1]] <- list("No" = individual, "Meadow" = unlist(tttt$MeadowName),
                "dispersed" = dispersed, "homeMeadow" = dispersed.from, "Distance" = distance, "dates" = tttt$Date, "sex" = tttt$Sex[1],
                "MownAt" = tttt$MownAt[1])
        }
    }
    return(recaptures)
}

getDispersed <- function(x) {
    dispersed <- matrix(ncol = 10, nrow = 0)
    for (i in 1:length(x)) {
        for (j in 1:length(x[[i]]$dispersed)) {
            Sex <- dataset[dataset$IndividualName == x[[i]]$No, ]$Sex[1]
            dispersed <- rbind(dispersed,
                    cbind(x[[i]]$No, Sex, x[[i]]$homeMeadow[j],
                    x[[i]]$Meadow[j], x[[i]]$Meadow[(j + 1)],
                    x[[i]]$dispersed[j],
                    as.character(x[[i]]$dates[j]), as.character(x[[i]]$dates[(j + 1)]),
                    x[[i]]$MownAt[1], x[[i]]$Distance[j]))
        }
    }
    dispersed <- as.data.frame(dispersed)
    colnames(dispersed) <- c("IndividualNo", "Sex", "HomeMeadow", "CapturedAt", "RecapturedAt", "Dispersed", "DateCapture", "DateRecapture", "MownAt", "Distance")
    dispersed$Distance <- round(as.numeric(dispersed$Distance), digits = 3)
    return(dispersed)
}

getReturns <- function(x) {
    recaptures <- getRecaptures(x)
    returns <- matrix(nrow = 0, ncol = 5)
    for (i in 1:length(recaptures)) {
        if (length(recaptures[[i]]$Meadow) > 2 ) {
            if ("Yes" %in% recaptures[[i]]$dispersed) {
                if (recaptures[[i]]$Meadow[1] == recaptures[[i]]$Meadow[length(recaptures[[i]]$Meadow)]) {
                    print(paste0("Individual", recaptures[[i]]$No, " returned home to ", recaptures[[i]]$Meadow[1]))
                    returns <- rbind(returns,
                        cbind(recaptures[[i]]$Meadow[1], recaptures[[i]]$No, "Yes", x[x$MeadowName == recaptures[[i]]$Meadow[1], ]$MownAt[1], recaptures[[i]]$sex))
                } else {
                    returns <- rbind(returns,
                        cbind(recaptures[[i]]$Meadow[1], recaptures[[i]]$No, "No", x[x$MeadowName == recaptures[[i]]$Meadow[1], ]$MownAt[1], recaptures[[i]]$sex))
                }
            }
        }
    }
    returns <- as.data.frame(returns)
    colnames(returns) <- c("MeadowName", "No", "Returns", "MownAt", "Sex")
    return(returns)
}

dispersed <- getDispersed(getRecaptures(dataset))

emigration <- matrix(nrow = length(unique(dispersed$CapturedAt)), ncol = 5)
for (i in 1:length(unique(dispersed$CapturedAt))) {
    meadow <- unique(dispersed$CapturedAt)[i]
    tttt <- dispersed[dispersed$CapturedAt == meadow, ]
    d <- nrow(tttt[tttt$Dispersed == "Yes", ]) # Number of dispersed individuals
    mownText <- tttt$MownAt[1]
    #mownText <- dataset[dataset$MeadowName == meadow, ]$MownAt[1]
    print(paste0(meadow, "; mown at: ", mownText, "; emigration: ", (d / nrow(tttt)) ))
    emigration[i, 1] <- meadow
    emigration[i, 2] <- mownText
    emigration[i, 3] <- d / nrow(tttt)
    emigration[i, 4] <- d
    emigration[i, 5] <- nrow(tttt)
}
emigration <- as.data.frame(emigration)
colnames(emigration) <- c("MeadowName", "MownAt", "Emigration", "Dispersed", "DisplacedAndDispersed")

emigration$Emigration <- as.numeric(emigration$Emigration)

# Get the survival
getSurvival <- function(est, d) {
    est.phi <- c()
    mown <- c()
    sex <- c()
    meadow <- c()

    for (i in 1:length(est)) {
        estimate <- est[[i]]

        if (length(estimate) == 0) {
            print("Skip this")
        } else {
            if (length(estimate$best_model$Phi) == 0) {
                print("Skip this")
            } else {
                meadowName <- estimate$MeadowName
                mownAt <- d[d$MeadowName == meadowName, ]$MownAt[1]
                print(paste0("Meadow ", meadowName, " mown at ", mownAt, " (", nrow(estimate$best_model$Phi), " parameters)"))
                if (nrow(estimate$best_model$Phi) == 1 ) {
                    est.phi <- c(est.phi, estimate$best_model$Phi$estimate)
                    mown <- c(mown, mownAt)
                    sex <- c(sex, "Male")
                    meadow <- c(meadow, meadowName)
                    est.phi <- c(est.phi, estimate$best_model$Phi$estimate)
                    mown <- c(mown, mownAt)
                    sex <- c(sex, "Female")
                    meadow <- c(meadow, meadowName)
                }
                if (nrow(estimate$best_model$Phi) == 2 ) {
                    est.phi <- c(est.phi, estimate$best_model$Phi$estimate[1])
                    mown <- c(mown, mownAt)
                    sex <- c(sex, "Male")
                    meadow <- c(meadow, meadowName)
                    est.phi <- c(est.phi, estimate$best_model$Phi$estimate[2])
                    mown <- c(mown, mownAt)
                    sex <- c(sex, "Female")
                    meadow <- c(meadow, meadowName)
                }

            }
        }
    }
    out <- cbind(est.phi, mown, sex, meadow)
    out <- as.data.frame(out)
    colnames(out) <- c("Phi", "MownAt", "Sex", "MeadowName")
    return(out)
}

survival <- getSurvival(estimates.mrr, dataset)
survival[survival$MownAt == "17.7.2014", ]$MownAt <- "Recovered"
survival[survival$MownAt != "Unmown" & survival$MownAt != "Recovered", ]$MownAt <- "Mown"
survival$Phi <- as.numeric(survival$Phi)

# Get the capture probability
getCaptureProbability <- function(est, d) {
    est.phi <- c()
    mown <- c()
    sex <- c()
    meadow <- c()

    for (i in 1:length(est)) {
        estimate <- est[[i]]

        if (length(estimate) == 0) {
            print("Skip this")
        } else {
            if (length(estimate$best_model$p) == 0) {
                print("Skip this")
            } else {
                meadowName <- estimate$MeadowName
                mownAt <- d[d$MeadowName == meadowName, ]$MownAt[1]
                print(paste0("Meadow ", meadowName, " mown at ", mownAt, " (", nrow(estimate$best_model$p), " parameters)"))
                if (nrow(estimate$best_model$p) == 1 ) {
                    est.phi <- c(est.phi, estimate$best_model$p$estimate)
                    mown <- c(mown, mownAt)
                    sex <- c(sex, "Male")
                    meadow <- c(meadow, meadowName)
                    est.phi <- c(est.phi, estimate$best_model$p$estimate)
                    mown <- c(mown, mownAt)
                    sex <- c(sex, "Female")
                    meadow <- c(meadow, meadowName)
                }
                if (nrow(estimate$best_model$p) == 2 ) {
                    est.phi <- c(est.phi, estimate$best_model$p$estimate[1])
                    mown <- c(mown, mownAt)
                    sex <- c(sex, "Male")
                    meadow <- c(meadow, meadowName)
                    est.phi <- c(est.phi, estimate$best_model$p$estimate[2])
                    mown <- c(mown, mownAt)
                    sex <- c(sex, "Female")
                    meadow <- c(meadow, meadowName)
                }

            }
        }
    }
    out <- cbind(est.phi, mown, sex)
    out <- as.data.frame(out)
    colnames(out) <- c("p", "MownAt", "Sex")
    return(out)
}

capture <- getCaptureProbability(estimates.mrr, dataset)
capture[capture$MownAt == "17.7.2014", ]$MownAt <- "Recovered"
capture[capture$MownAt != "Unmown" & capture$MownAt != "Recovered", ]$MownAt <- "Mown"
capture$p <- as.numeric(capture$p)

#############################################
# Prepare other variables for GLM models
#############################################

library(emmeans)
library(lme4)

# Sort out the mowing regimes
tttt <- dataset
tttt[is.na(tttt$ExactMowing), ]$ExactMowing <- as.Date("2014-11-1") # Temporary assign the date into the future
tttt$Regrow <- tttt$ExactMowing + 15
tttt[(tttt$Date >= tttt$ExactMowing) & (tttt$Date <= tttt$Regrow), ]$MownAt <- "Mown" # The dates where Sanguissorba officinalis is not present
tttt[(tttt$Date < tttt$ExactMowing) & (tttt$MownAt != "Recovered"), ]$MownAt <- "Unmown" # The meadows were unmown before mowing
tttt[(tttt$Date > tttt$Regrow), ]$MownAt <- "Recovered" # After 15 days, the meadows are considered recovered
tttt[tttt$ExactMowing == as.Date("2014-11-1"), ]$ExactMowing <- NA # Get the date back to the original NA value

dispersed <- getDispersed(getRecaptures(tttt))

imigrationIndex <- function(meadows) {
    centroids <- gCentroid(meadows, byid = T)
    centroids <- SpatialPoints(centroids)
    centroids <- SpatialPointsDataFrame(centroids, meadows@data)

    alpha <- 1 / (mean(dispersed[dispersed$Dispersed == "Yes", ]$Distance) / 1000)
    Zim <- 0.5
    connectivity.index <- matrix(nrow = 0, ncol = 2)
    for (i in 1:nrow(centroids)) {
        conect <- NA
        meadow.name <- meadows@data[i, ]$MeadowName
        meadow.coords <- centroids[centroids@data$MeadowName == meadow.name, ]
        buffer <- gBuffer(meadow.coords, width = 3000)
        proj4string(buffer) <- CRS("+init=epsg:3035")

        subseter <- over(meadows[meadows@data$MeadowName != meadow.name, ], buffer) == 1
        subseter[is.na(subseter)] <- FALSE
        neighbours <- meadows[subseter, ]

        for (j in 1:nrow(neighbours)) {
            distance <- gDistance(meadows[meadows@data$MeadowName == meadow.name, ], neighbours[j, ])
            Area <- neighbours[j, ]$Area
            conect <- c(conect, exp(-alpha * distance) * Area ^ Zim)
        }
        print(paste0("Meadow ", meadow.name, " connectivity index is ", sum(conect, na.rm = T), " (", nrow(neighbours), " neighbours)"))
        connectivity.index <- rbind(connectivity.index, cbind(meadow.name, sum(conect, na.rm = T)))
    }
    connectivity.index <- as.data.frame(connectivity.index)
    colnames(connectivity.index) <- c("MeadowName", "ImigrationChance")
    return(connectivity.index)
}

im.index <- imigrationIndex(meadows)

emigrationIndex <- function(meadows) {

    eta <- 0.0789
    Zem <- -0.4926
    emigration.index <- matrix(nrow = 0, ncol = 2)
    for (i in 1:nrow(meadows)) {
        meadow <- meadows[i, ]

        area <- meadow$Area
        emigration <- eta * area ^ -Zem

        print(paste0("Meadow ", meadow$MeadowName, " connectivity index is ", emigration))
        emigration.index <- rbind(emigration.index, cbind(meadow$MeadowName, emigration))
    }
    emigration.index <- as.data.frame(emigration.index)
    colnames(emigration.index) <- c("MeadowName", "EmigrationRate")
    return(emigration.index)
}

em.index <- emigrationIndex(meadows)

# Get the number of captures per meadow
dispersed$NoCaptures <- NA
dispersed$PopulationSize <- NA
dispersed$Locality <- NA
for (i in 1:length(unique(dispersed$CapturedAt))) {
    meadowName <- unique(dispersed$CapturedAt)[i]
    numberOfCaptures <- length(dataset[dataset$MeadowName == meadowName, ]$IndividualName)
    locality <- dataset[dataset$MeadowName == meadowName, ]$Locality[1]
    dispersed[dispersed$CapturedAt == meadowName, ]$NoCaptures <- numberOfCaptures
    dispersed[dispersed$CapturedAt == meadowName, ]$Locality <- locality
    dispersed[dispersed$CapturedAt == meadowName, ]$PopulationSize <-
        recruitment[[which(unlist(lapply(recruitment, function(x) {x$MeadowName == meadowName})))]]$TotalPopulation
}

# Get the survival and capture probability
getSurvivalAndCaptureProbability <- function(d, est) {
    est.phi <- c()
    est.p <- c()
    for (i in 1:nrow(d)) {
        meadowName <- d[i, ]$CapturedAt
        sex <- d[i, ]$Sex
        no <- which(unlist(lapply(est, function(x) {x$MeadowName == meadowName})))

        if (length(no) == 0) {
            est.phi <- c(est.phi, NA)
            est.p <- c(est.p, NA)
        }

        else {
            if (length(est[[no]]$best_model$Phi) == 0) {
                est.phi <- c(est.phi, NA)
                est.p <- c(est.p, NA)
            } else {
                if (nrow(est[[no]]$best_model$Phi) == 1) {
                    est.phi <- c(est.phi, est[[no]]$best_model$Phi$estimate)
                }
                if (nrow(est[[no]]$best_model$Phi) == 2) {
                    if (sex == "Male") {
                        est.phi <- c(est.phi, est[[no]]$best_model$Phi$estimate[1])
                    } else {
                        est.phi <- c(est.phi, est[[no]]$best_model$Phi$estimate[2])
                    }
                }

                if (nrow(est[[no]]$best_model$p) == 1) {
                    est.p <- c(est.p, est[[no]]$best_model$p$estimate)
                }
                if (nrow(est[[no]]$best_model$p) == 2) {
                    if (sex == "Male") {
                        est.p <- c(est.p, est[[no]]$best_model$p$estimate[1])
                    } else {
                        est.p <- c(est.p, est[[no]]$best_model$p$estimate[2])
                    }
                }
            }

        }
    }
    out <- cbind(est.phi, est.p)
    out <- as.data.frame(out)
    colnames(out) <- c("Phi", "p")
    return(out)
}

dispersed <- cbind(dispersed, getSurvivalAndCaptureProbability(dispersed, estimates.mrr))

# Create a few more variables used in the model
dispersed$Dispersed  <- dispersed$Dispersed == "Yes"
dispersed$Area <- NA
dispersed$ImigrationChance <- NA
dispersed$EmigrationRate <- NA
for (i in 1:nrow(dispersed)) {
    dispersed$Area[i] <- meadows@data[meadows@data$MeadowName == dispersed$CapturedAt[i], ]$Area
    dispersed$ImigrationChance[i] <- as.numeric(as.character( im.index[im.index$MeadowName == dispersed$CapturedAt[i], ]$ImigrationChance ))
    dispersed$EmigrationRate[i] <- as.numeric(as.character( em.index[em.index$MeadowName == dispersed$CapturedAt[i], ]$EmigrationRate))
}

############################################################
# Model survival, capture probability
# and population size usin GLM
############################################################

dataset.tttt <- cbind(capture, survival)[,c(-2, -3)]
dataset.tttt$Area <- NA
dataset.tttt$PopulationSize <- NA
dataset.tttt$Connectivity <- NA
dataset.tttt$Locality <- NA
for (i in 1:nrow(dataset.tttt)) {
    dataset.tttt[i, ]$Area <- meadows[meadows@data$MeadowName == dataset.tttt[i, ]$MeadowName, ]@data$Area
    dataset.tttt[i, ]$PopulationSize <- recruitment[[which(unlist(lapply(recruitment, function(x) {x$MeadowName == dataset.tttt[i, ]$MeadowName})))]]$TotalPopulation
    dataset.tttt[i, ]$Connectivity <- as.numeric(as.character( im.index[im.index$MeadowName == dataset.tttt[i, ]$MeadowName, ]$ImigrationChance ))
    dataset.tttt[i, ]$Locality <- dataset[dataset$MeadowName == dataset.tttt[i, ]$MeadowName, ]$Locality[1]
}
dataset.tttt$MownAt <- relevel(as.factor(dataset.tttt$MownAt), "Recovered")
dataset.tttt$MownAt <- relevel(as.factor(dataset.tttt$MownAt), "Unmown")

model.combinations <- function(x, depVar) {
    models <- list()
    models <- c(models, as.formula(paste0(depVar, " ~ 1 ", "+ (1|Locality)")))
    for (i in 1:length(x)) {
        vc <- combn(x, i)
        for (j in 1:ncol(vc)) {
            model <- as.formula(paste0(depVar, " ~ ", paste0(vc[, j], collapse = " + "), "+ (1|Locality)"))
            models <- c(models, model)
        }
    }
    return(models)
}

############################################################
# Model 1: Survival
dataset.tttt$Density <- dataset.tttt$PopulationSize / (dataset.tttt$Area / 10000)

models <- model.combinations(c("MownAt", "Sex", "scale(Area)", "scale(PopulationSize)", "scale(Connectivity)", "scale(Area):scale(Connectivity)"), "Phi")
models.aic <- matrix(ncol = 2, nrow=length(models))
for (i in 1:length(models)) {
    lmermodel <- lmer(models[[i]], data = dataset.tttt)
    models.aic[i, 1] <- as.character(models[[i]][3])
    models.aic[i, 2] <- AIC(lmermodel)
}
models.aic[order(as.numeric(as.character(models.aic[, 2]))), ]

############################################################
# Model 2: Capture Probability
models <- model.combinations(c("MownAt", "Sex", "scale(Area)", "scale(PopulationSize)", "scale(Connectivity)", "scale(Area):scale(Connectivity)"), "p")
models.aic <- matrix(ncol = 2, nrow=length(models))
for (i in 1:length(models)) {
    lmermodel <- lmer(models[[i]], data = dataset.tttt)
    models.aic[i, 1] <- as.character(models[[i]][3])
    models.aic[i, 2] <- AIC(lmermodel)
}
models.aic[order(as.numeric(as.character(models.aic[, 2]))), ]

############################################################
# Model 3: Population Size
models <- model.combinations(c("MownAt", "scale(Area)", "scale(Connectivity)", "scale(Area):scale(Connectivity)"), "PopulationSize")
models.aic <- matrix(ncol = 2, nrow=length(models))
for (i in 1:length(models)) {
    lmermodel <- glmer(models[[i]], data = dataset.tttt[dataset.tttt$Sex == "Male", ], family = poisson())
    models.aic[i, 1] <- as.character(models[[i]][3])
    models.aic[i, 2] <- AIC(lmermodel)
}
models.aic[order(as.numeric(as.character(models.aic[, 2]))), ]
lmermodel <- glmer(PopulationSize ~ MownAt + scale(Area) + scale(Connectivity) + scale(Area):scale(Connectivity) + (1|Locality), data = dataset.tttt[dataset.tttt$Sex == "Male", ], family = poisson())
emmeans.population <- emmeans(lmermodel, pairwise ~ MownAt + Area + Connectivity, type = "response")
summary(lmermodel)
summary(emmeans.population)

barmean <- barplot(summary(emmeans.population)$emmeans[, 4], ylim = c(0,400), col = c("#00a000", "#b7d300", "#c3051b"))
arrows(barmean, summary(emmeans.population)$emmeans[, 4] - summary(emmeans.population)$emmeans[, 5],
    barmean, summary(emmeans.population)$emmeans[, 4] + summary(emmeans.population)$emmeans[, 5],
    lwd = 1.5, angle = 90, code = 3, length = 0.05)

###################################
# Model 4: probability of emigration

dispersed$MownAt <- relevel(as.factor(dispersed$MownAt), "Recovered")
dispersed$MownAt <- relevel(dispersed$MownAt, "Unmown")

models <- model.combinations(c("MownAt", "Sex", "scale(PopulationSize)", "scale(ImigrationChance)", "scale(Area)", "scale(PopulationSize):scale(ImigrationChance):scale(Area)", "scale(PopulationSize):scale(ImigrationChance)", "scale(PopulationSize):scale(Area)"), "Dispersed")
models.aic <- matrix(ncol = 2, nrow=length(models))
for (i in 1:length(models)) {
    dispersal.probability <- glmer(models[[i]], data = dispersed, family = binomial)
    models.aic[i, 1] <- as.character(models[[i]][3])
    models.aic[i, 2] <- AIC(dispersal.probability)
}
models.aic[order(models.aic[, 2]), ][1:15]

dispersal.prob.fit <- glmer(Dispersed ~ MownAt + Sex + scale(PopulationSize) + scale(ImigrationChance) + scale(PopulationSize):scale(ImigrationChance):scale(Area) + scale(PopulationSize):scale(ImigrationChance) + (1|Locality), data = dispersed, family = binomial) # AIC = 1637.033

emm <- emmeans(dispersal.prob.fit, pairwise ~ MownAt + Sex + PopulationSize + ImigrationChance + Area, type = "response")
summary(dispersal.prob.fit)
summary(emm)

barmean <- barplot(summary(emm)$emmeans[,6], ylim = c(0,0.5), col = c("#00a000", "#b7d300", "#c3051b", "#00a000", "#b7d300", "#c3051b"))
arrows(barmean, summary(emm)$emmeans[,6] - summary(emm)$emmeans[, 7],
    barmean, summary(emm)$emmeans[,6] + summary(emm)$emmeans[, 7],
    lwd = 1.5, angle = 90, code = 3, length = 0.05)

###################################
# Model 5: The distance of dispersal

data.dispersed <- dispersed[dispersed$Dispersed == TRUE, ]
data.dispersed <- data.dispersed[!is.na(data.dispersed$PopulationSize), ]
data.dispersed$MownAt <- relevel(as.factor(data.dispersed$MownAt), "Recovered")
data.dispersed$MownAt <- relevel(data.dispersed$MownAt, "Unmown")

models <- model.combinations(c("MownAt", "Sex", "scale(PopulationSize)", "scale(ImigrationChance)", "scale(Area)", "scale(PopulationSize):scale(ImigrationChance):scale(Area)", "scale(PopulationSize):scale(ImigrationChance)", "scale(PopulationSize):scale(Area)"), "Distance")
#models <- model.combinations(c("MownAt", "Sex", "PopulationSizeS", "ImigrationChanceS", "AreaS"), "Distance")
models.aic <- matrix(ncol = 2, nrow=length(models))
for (i in 1:length(models)) {
    dispersal.probability <- glmer(models[[i]], data = data.dispersed, family = Gamma(link = "log")) # AIC = 1650.2
    models.aic[i, 1] <- as.character(models[[i]][3])
    models.aic[i, 2] <- AIC(dispersal.probability)
}
models.aic[order(models.aic[, 2]), ][1:63, ]

dispersal.dist.fit <- glmer(Distance ~ MownAt + Sex + scale(Area) + (1|Locality), data = data.dispersed, family = Gamma(link = "log"), na.action = )

summary(dispersal.dist.fit)
emm <- emmeans(dispersal.dist.fit, pairwise ~ MownAt + Sex + Area, type = "response")

barmean <- barplot(summary(emm)$emmeans[,4], ylim = c(0,400), col = c("#00a000", "#b7d300", "#c3051b", "#00a000", "#b7d300", "#c3051b"))
arrows(barmean, summary(emm)$emmeans[,4] - summary(emm)$emmeans[, 5],
    barmean, summary(emm)$emmeans[,4] + summary(emm)$emmeans[, 5],
    lwd = 1.5, angle = 90, code = 3, length = 0.05)

###################################
# Model 6: The distance of displacement

data.displaced <- dispersed[dispersed$Dispersed == FALSE, ]

#models <- model.combinations(c("MownAt", "Sex", "PopulationSizeS", "AreaS"), "Distance")
models <- model.combinations(c("MownAt", "Sex", "scale(PopulationSize)", "scale(ImigrationChance)", "scale(Area)", "scale(PopulationSize):scale(ImigrationChance):scale(Area)", "scale(PopulationSize):scale(ImigrationChance)", "scale(PopulationSize):scale(Area)"), "Distance")
models.aic <- matrix(ncol = 2, nrow=length(models))
for (i in 1:length(models)) {
    dispersal.probability <- glmer(models[[i]], data = data.displaced , family = Gamma(link = "log"))
    models.aic[i, 1] <- as.character(models[[i]][3])
    models.aic[i, 2] <- AIC(dispersal.probability)
}
models.aic[order(models.aic[, 2]), ][1:54, ]

displacement.fit <- glmer(Distance ~ Sex + scale(PopulationSize) + scale(Area):scale(PopulationSize) + (1|Locality), data = data.displaced , family = Gamma(link = "log")) # AIC = 11159

summary(displacement.fit)
emm <- emmeans(displacement.fit, pairwise ~ Sex + PopulationSize + Area, type = "response")

barmean <- barplot(summary(emm)$emmeans[,4], ylim = c(0,70), col = c("#ffffff"))
arrows(barmean, summary(emm)$emmeans[, 4] + summary(emm)$emmeans[, 5],
    barmean, summary(emm)$emmeans[, 4] - summary(emm)$emmeans[, 5],
    lwd = 1.5, angle = 90, code = 3, length = 0.05)


############################
# write the dateset for supplementary
write.csv(dataset[, -c(6, 7, 8, 11)], file = "dataset.csv", row.names = F)
